﻿package {
	import flash.display.*;
	import flash.display.BitmapData;
	import flash.display.BitmapDataChannel;
	import flash.display.BlendMode;
	import flash.display.DisplayObject;
	import flash.display.DisplayObjectContainer;
	import flash.geom.Matrix;
	import flash.geom.Point;
	import flash.geom.Rectangle;
	public class PixelPerfectCollisionDetection {
/** Get the collision rectangle between two display objects. **/
		public static function getCollisionRect(target1:DisplayObject, target2:DisplayObject, commonParent:DisplayObjectContainer, pixelPrecise:Boolean = false, tolerance:Number = 0, pXDisp:Number=0, pYDisp:Number=0):Rectangle {
			// get bounding boxes in common parent's coordinate space
			var rect1:Rectangle=target1.getBounds(commonParent);
			var rect2:Rectangle=target2.getBounds(commonParent);
			rect1.x += pXDisp;
			rect1.y += pYDisp;
			// find the intersection of the two bounding boxes
			var intersectionRect:Rectangle=rect1.intersection(rect2);
			if (intersectionRect.size.length>0) {
				if (pixelPrecise) {
					// size of rect needs to integer size for bitmap data
					intersectionRect.width=Math.ceil(intersectionRect.width);
					intersectionRect.height=Math.ceil(intersectionRect.height);
					// get the alpha maps for the display objects
					var alpha1:BitmapData=getAlphaMap(target1,intersectionRect,BitmapDataChannel.RED,commonParent,pXDisp,pYDisp);
					var alpha2:BitmapData=getAlphaMap(target2,intersectionRect,BitmapDataChannel.GREEN,commonParent);
					//Game.getGame().drawCollision1(alpha1.clone());
					//Game.getGame().drawCollision2(alpha2);
					// combine the alpha maps
					alpha1.draw(alpha2, null, null, BlendMode.LIGHTEN);
					// calculate the search color
					var searchColor:uint;
					if (tolerance<=0) {
						searchColor=0x010000;
					}
					else {
						if (tolerance>1) {//Tolerance=0=Responds to any alpha, 1=Does not respond to any alpha
							tolerance=1;
						}
						var byte:int=Math.round(tolerance*255);
						searchColor = (byte <<16) | (byte <<8) | 0;
					}
					// find color
					var collisionRect:Rectangle=alpha1.getColorBoundsRect(searchColor,searchColor);
					//Game.getGame().drawCollision0(alpha1);
					if (collisionRect.width<=0||collisionRect.height<=0) {
						return null;
					}
					collisionRect.x+=intersectionRect.x;
					collisionRect.y+=intersectionRect.y;
					return collisionRect;
				}
				else {
					return intersectionRect;
				}
			}
			else {
				//Game.getGame().drawCollision0(new BitmapData(10,10));
				//Game.getGame().drawCollision1(new BitmapData(10,10));
				//Game.getGame().drawCollision2(new BitmapData(10,10));
				// no intersection
				return null;
			}
		}
/** Gets the alpha map of the display object and places it in the specified channel. **/
		private static function getAlphaMap(target:DisplayObject, rect:Rectangle, channel:uint, commonParent:DisplayObjectContainer, pXDisp:Number=0, pYDisp:Number=0):BitmapData {
			// calculate the transform for the display object relative to the common parent
			var fParentDO:DisplayObject=target.parent;
			//CANNOT USE transform.concatinatedMatrix if stage is zoomed,there is a cacheAsBitmap anywhere in hierarchy
			//Therefore must manually concatinate every matrix in hierarchy
			var targetXform:Matrix=target.transform.matrix.clone();
			if(commonParent.contains(target)){
				while(fParentDO!=commonParent){
					var fParentXform:Matrix=fParentDO.transform.matrix.clone();
					targetXform.concat(fParentXform);
					fParentDO = fParentDO.parent;
				}
			}
			var parentXformInvert:Matrix=commonParent.transform.matrix.clone();
			parentXformInvert.invert();
			targetXform.concat(parentXformInvert);
//			// translate the target into the rect's space
			targetXform.translate(-rect.x+pXDisp, -rect.y+pYDisp);

			// draw the target and extract its alpha channel into a color channel
			var bitmapData:BitmapData=new BitmapData(rect.width,rect.height,true,0);
			bitmapData.draw(target, targetXform);
			var alphaChannel:BitmapData=new BitmapData(rect.width,rect.height,false,0);
			alphaChannel.copyChannel(bitmapData, bitmapData.rect, new Point(0, 0), BitmapDataChannel.ALPHA, channel);

			return alphaChannel;
		}

/** Get the center of the collision's bounding box. **/
		public static function getCollisionPoint(target1:DisplayObject, target2:DisplayObject, commonParent:DisplayObjectContainer, pixelPrecise:Boolean = false, tolerance:Number = 0):Point {
			var collisionRect:Rectangle=getCollisionRect(target1,target2,commonParent,pixelPrecise,tolerance);
			if (collisionRect!=null&&collisionRect.size.length>0) {
				var x:Number = (collisionRect.left + collisionRect.right) / 2;
				var y:Number = (collisionRect.top + collisionRect.bottom) / 2;
				return new Point(x, y);
			}

			return null;
		}

/** Are the two display objects colliding (overlapping)? **/
		public static function isColliding(target1:DisplayObject, target2:DisplayObject, commonParent:DisplayObjectContainer, pixelPrecise:Boolean = false, tolerance:Number = 0, pXDisp:Number=0, pYDisp:Number=0):Boolean {
			var collisionRect:Rectangle=getCollisionRect(target1,target2,commonParent,pixelPrecise,tolerance,pXDisp,pYDisp);
			if (collisionRect!=null&&collisionRect.size.length>0) {
				return true;
			}
			else {
				return false;
			}
		}
	}
}
