﻿package {
	import flash.display.*;
	import flash.events.*;
	import flash.utils.getDefinitionByName;
	import flash.media.Sound;
	import flash.media.SoundTransform;
	import flash.media.SoundChannel;
	import flash.utils.getTimer;

	public class SoundEffect extends MovieClip{
		private var sound:Sound = new Sound();
		private var soundName:String = null;
		private var vol:Number = 1;
		private var volCurrent:Number = 1;
		private var soundTrans:SoundTransform = new SoundTransform();
		private var currentTimeFunc:Function = null;
		private var containerArray:Array;
		
		//INTRO LOOP
		private var introLoopOn:Boolean = false;
		private var loopSoundName:String = null
		private var loopSound:Sound;
		
		//LOOP
		private var soundLoopingOn:Boolean = false;
		private var soundChannel:SoundChannel;
		private var startLoopTime:uint;
		private var numLoops:uint;
		private var numLoopsCounter:uint;
		private var endlessOn:Boolean;

		//ECHO
		private var echoOn:Boolean = false;
		private var echoInterval:uint = 400;//Amount of time b/t each echo
		private var echoAmount:uint = 3;//Number of echos
		private var echoAmountCurrent:uint = 0;
		private var echoReduction:Number = .6;//Each subsequent echo is ~ of previous echo volume
		private var echoSoundArray:Array = new Array(0);
		private var echoFadeOutOn:Boolean = true;
		private var echoTotalTimePassed:uint = 0;
		private var lastTimeEcho:uint;
		private var soundEffectComplete:Boolean = false;
		
		//FADE OUT
		private var fadeOutDuration:uint;
		private var lastTimeFadeOut:uint;
		
		//FADE IN
		private var fadeInDuration:uint;
		private var lastTimeFadeIn:uint;
		private var fadeInStartVol:Number;
		private var fadeInEndVol:Number;
		
		private var tempVolArray:Array = new Array(0);
		private var tempVol:Number = 0;
		
		public function SoundEffect(pSoundName:String, pVol:Number, pCurrentTimeFunc:Function, pContainerArray:Array){
			if(pSoundName!=null){
				pContainerArray.push(this);
				var FSoundNameClass:Class = getDefinitionByName(pSoundName) as Class;
				var fSound:Sound = new FSoundNameClass();
				sound = fSound;
				soundName = pSoundName;
				vol = pVol;
				volCurrent = pVol;
				soundTrans.volume = pVol;
				currentTimeFunc = pCurrentTimeFunc;
				containerArray = pContainerArray;
			}
		}
//		public function setEcho(pEchoOn:Boolean, pEchoInterval:uint, pEchoAmount:uint, pEchoReduction:Number){
//			if(pEchoOn){
//				echoOn = true;
//				echoInterval = pEchoInterval;
//				echoAmount = pEchoAmount;
//				echoReduction = pEchoReduction;
//			}
//		}
		public function startSoundEffect(){
			try{
				soundChannel = sound.play(0,0,soundTrans);
				soundChannel.addEventListener(Event.SOUND_COMPLETE, removeSoundEffectEventFunc);
			}
			catch(errObj:Error){
				trace("No Sound?", errObj.message);
				return;
			}
		}
		public function startEchoSoundEffect(pEchoInterval:uint, pEchoAmount:int, pEchoReduction:Number){
				echoOn = true;
				echoInterval = pEchoInterval;
				echoAmount = pEchoAmount;
				echoReduction = pEchoReduction;
				
				echoTotalTimePassed = 0;
				
				if(echoAmount>0 || echoAmount==-1){
					var FSoundNameClass:Class = getDefinitionByName(soundName) as Class;
					var fSound:Sound = new FSoundNameClass();
					var fSoundVol:SoundTransform = new SoundTransform(vol);
					var fSoundChannel:SoundChannel = fSound.play(0,0,fSoundVol);
					echoAmountCurrent = 1;//Start at 1 since we play once at start and for timing
					volCurrent = vol;
					echoSoundArray = [fSoundChannel, fSoundVol, volCurrent, fSound.length, currentTimeFunc()];
					lastTimeEcho = currentTimeFunc();//To start first time
					addEventListener(Event.ENTER_FRAME, echoLoopEvent);
				}
		}
		private function echoLoopEvent(e:Event){
			echoLoopFunc();
		}
		private function echoLoopFunc(){
			if(echoAmountCurrent<echoAmount || echoAmount==-1){
				var fTimePassed:uint = currentTimeFunc()-lastTimeEcho;
				echoTotalTimePassed += fTimePassed;
				lastTimeEcho += fTimePassed;
				if(echoTotalTimePassed>=echoInterval){
					echoTotalTimePassed = echoTotalTimePassed-echoInterval;
					var FSoundNameClass:Class = getDefinitionByName(soundName) as Class;
					var fSound:Sound = new FSoundNameClass();
					var fSoundVol:SoundTransform = new SoundTransform(volCurrent);
					var fSoundChannel:SoundChannel = fSound.play(0,0,fSoundVol);
					volCurrent *= echoReduction;
					echoSoundArray.push(fSoundChannel, fSoundVol, volCurrent, fSound.length, currentTimeFunc());
					echoAmountCurrent++;
				}
				if(echoFadeOutOn){
					for(var i:uint=0; i<echoSoundArray.length; i+=5){
						var fVol:Number = echoSoundArray[i+1].volume;
						var fSoundCompletion:Number = (currentTimeFunc()-echoSoundArray[i+4])/echoSoundArray[i+3];
						fVol = echoSoundArray[i+2]*(1-fSoundCompletion);
						if(fVol<0){
							fVol=0;
						}
						echoSoundArray[i+1].volume = fVol;
						echoSoundArray[i].soundTransform = echoSoundArray[i+1];
					}
				}
			}
			else{
				removeEventListener(Event.ENTER_FRAME, echoLoopEvent);
				removeSoundEffect();
				soundEffectComplete = true;
			}
		}
		public function startIntroLoopSoundEffect(pLoopSoundName:String){
			introLoopOn = true;
			soundChannel = sound.play(0,0,soundTrans);
			soundChannel.addEventListener(Event.SOUND_COMPLETE, introLoopSound);
			
			if(pLoopSoundName!=null){
				var FSoundNameClass:Class = getDefinitionByName(pLoopSoundName) as Class;
				var fSound:Sound = new FSoundNameClass();
				loopSoundName = pLoopSoundName;
				loopSound = fSound;
			}
			else{
				loopSoundName = soundName;
				loopSound = sound;
			}
		}
		private function introLoopSound(e:Event){
				soundChannel.removeEventListener(Event.SOUND_COMPLETE, introLoopSound);
				soundChannel = loopSound.play(0,0,soundTrans);
				soundChannel.addEventListener(Event.SOUND_COMPLETE, introLoopSound);
		}
		public function startLoopingSoundEffect(fStartTime:uint, fStartLoopTime:uint, fNumLoops:uint, fEndlessOn:Boolean){
			soundLoopingOn = true;
			var fSoundChannel:SoundChannel = sound.play(fStartTime,0,soundTrans);
			soundChannel = fSoundChannel;
			startLoopTime = fStartLoopTime;
			numLoops = fNumLoops;
			numLoopsCounter = fNumLoops;
			endlessOn = fEndlessOn;
			soundChannel = fSoundChannel;
			fSoundChannel.addEventListener(Event.SOUND_COMPLETE, loopingSound);
		}
		private function loopingSound(e:Event){
			if(endlessOn){
				soundChannel.removeEventListener(Event.SOUND_COMPLETE, loopingSound);
				var fSoundChannelA:SoundChannel = sound.play(startLoopTime,0,soundTrans);
				soundChannel = fSoundChannelA;
				fSoundChannelA.addEventListener(Event.SOUND_COMPLETE, loopingSound);
			}
			else{
				if(numLoopsCounter > 0){
					soundChannel.removeEventListener(Event.SOUND_COMPLETE, loopingSound);
					var fSoundChannelB:SoundChannel = sound.play(startLoopTime,0,soundTrans);
					soundChannel = fSoundChannelB;
					fSoundChannelB.addEventListener(Event.SOUND_COMPLETE, loopingSound);
					numLoopsCounter--;
				}
				else{
					soundChannel.removeEventListener(Event.SOUND_COMPLETE, loopingSound);
					soundChannel.addEventListener(Event.SOUND_COMPLETE, removeSoundEffectEventFunc);
				}
			}
		}
		public function startFadeInSoundEffect(pDuration:uint, pStartVolume:Number, pEndVolume:Number){
			if(soundChannel!=null){//(since this is only set in start and loop) If sound effect has started or is looping (not echo)
				soundTrans.volume = pStartVolume;
				fadeInDuration = pDuration;
				fadeInStartVol = pStartVolume;
				fadeInEndVol = pEndVolume;
				lastTimeFadeIn = getTimer();
				addEventListener(Event.ENTER_FRAME, fadeInLoop);
				soundChannel.addEventListener(Event.SOUND_COMPLETE, removeSoundEffectEventFunc);
			}
		}
		private function fadeInLoop(e:Event){
			var fSoundCompletion:Number = (getTimer()-lastTimeFadeIn)/fadeInDuration;
			var fVol:Number = fadeInStartVol+(fadeInEndVol-fadeInStartVol)*(fSoundCompletion);
			if(fSoundCompletion>=1){
				fVol=fadeInStartVol+fadeInEndVol;
				vol = fVol;
				removeEventListener(Event.ENTER_FRAME, fadeInLoop);
			}
			volCurrent = fVol;
			soundTrans.volume = fVol;
			soundChannel.soundTransform = soundTrans;
		}
		public function startFadeOutSoundEffect(pDuration:uint){
			if(soundChannel!=null){//(since this is only set in start and loop) If sound effect has started or is looping (not echo)
				var fSoundTransform:SoundTransform = new SoundTransform(volCurrent);
				soundTrans = fSoundTransform;
				fadeOutDuration = pDuration;
				lastTimeFadeOut = getTimer();
				addEventListener(Event.ENTER_FRAME, fadeOutLoop);
				soundChannel.addEventListener(Event.SOUND_COMPLETE, removeSoundEffectEventFunc);
			}
		}
		private function fadeOutLoop(e:Event){
			var fSoundCompletion:Number = (getTimer()-lastTimeFadeOut)/fadeOutDuration;
			var fVol:Number = vol*(1-fSoundCompletion);
			if(fVol<=0){
				fVol=0;
				removeEventListener(Event.ENTER_FRAME, fadeOutLoop);
			}
			soundTrans.volume = fVol;
			soundChannel.soundTransform = soundTrans;
		}
		public function stopSoundEffect(pSoftStopOn:Boolean=false){//pSoftStopOn = Let it complete the last cycle if Echo (no immediate stop)
			if(soundLoopingOn){
				soundChannel.removeEventListener(Event.SOUND_COMPLETE, loopingSound);
				soundLoopingOn = false;
			}
			if(introLoopOn){
				soundChannel.removeEventListener(Event.SOUND_COMPLETE, introLoopSound);
				introLoopOn = false;
			}
			if(soundChannel!=null){
				soundChannel.stop();
			}
			if(echoOn){
				if(!pSoftStopOn){
					for(var i:uint=0; i<echoSoundArray.length; i+=5){
						echoSoundArray[i].stop();
					}
				}
				echoSoundArray = null;
				removeEventListener(Event.ENTER_FRAME, echoLoopEvent);
				echoOn = false;
				soundEffectComplete = true;
			}
		}
		public function setVolume(n:Number){
			vol = n;
			volCurrent = n;
			soundTrans.volume = n;
			soundChannel.soundTransform = soundTrans;
		}
		public function startTempVol(pTempVol:Number){
			tempVolArray.push(pTempVol);
			soundTrans.volume = pTempVol;
			soundChannel.soundTransform = soundTrans;
		}
		public function endTempVol(pTempVol:Number){
			ArrayHelper.findAndRemove(tempVolArray,pTempVol);
			if(tempVolArray.length==0){
				soundTrans.volume = vol;
			}
			else{
				soundTrans.volume = tempVolArray[tempVolArray.length-1];
			}
			soundChannel.soundTransform = soundTrans;
		}
		public function getVolCurrent(){
			return volCurrent;
		}
		public function getVol(){
			return vol;
		}
		public function getSoundName():String{
			return soundName;
		}
		public function getSoundEffectComplete():Boolean{
			return soundEffectComplete;
		}
		public function getEchoOn():Boolean{
			return echoOn;
		}
		private function removeSoundEffectEventFunc(e:Event){
			removeSoundEffect();
		}
		public function removeSoundEffect(){
			if(containerArray.indexOf(this)!=-1){
				containerArray.splice(containerArray.indexOf(this),1);
			}
			delete this;
		}
	}
}